@@ -186,11 +186,13 @@ int gauge(const char * text, int height, int width, poptContext optCon, int fd,
 	}
     } while (!feof(f));
 
+    newtFormDestroy(form);
+
     return DLG_OKAY;
 }
 
 int inputBox(const char * text, int height, int width, poptContext optCon, 
-		int flags, const char ** result) {
+		int flags, char ** result) {
     newtComponent form, entry, okay, cancel, answer, tb;
     const char * val;
     int pFlag = (flags & FLAG_PASSWORD) ? NEWT_FLAG_PASSWORD : 0;
@@ -210,12 +212,15 @@ int inputBox(const char * text, int height, int width, poptContext optCon,
     addButtons(height, width, form, &okay, &cancel, flags);
 
     answer = newtRunForm(form);
+    *result = NULL;
     if (answer == cancel)
 	rc = DLG_CANCEL;
     else if (answer == NULL)
 	rc = DLG_ESCAPE;
+    else
+	*result = strdup(val);
 
-    *result = val;
+    newtFormDestroy(form);
 
     return rc;
 }
@@ -250,7 +255,7 @@ static int mystrncpyw(char *dest, const char *src, int n, int *maxwidth)
 }
 
 int listBox(const char * text, int height, int width, poptContext optCon,
-		int flags, const char *default_item, const char ** result) {
+		int flags, const char *default_item, char ** result) {
     newtComponent form, okay, tb, answer, listBox;
     newtComponent cancel = NULL;
     const char * arg;
@@ -344,7 +349,8 @@ int listBox(const char * text, int height, int width, poptContext optCon,
 	   int len, j;
 	   len = mystrncpyw(buf, itemInfo[i].tag, MAXBUF, &w);
 	   for (j = 0; j < tagWidth - w; j++) {
-		   if (len >= MAXBUF) break;
+		   if (len + 1 >= MAXBUF)
+		       break;
 		   buf[len++] = ' ';
 	   }
 	   buf[len] = '\0';
@@ -367,19 +373,24 @@ int listBox(const char * text, int height, int width, poptContext optCon,
     addButtons(height, width, form, &okay, &cancel, flags);
 
     answer = newtRunForm(form);
+    *result = NULL;
     if (answer == cancel)
 	rc = DLG_CANCEL;
     if (answer == NULL)
 	rc = DLG_ESCAPE;
+    else {
+	i = (long) newtListboxGetCurrent(listBox);
+	*result = strdup(itemInfo[i].tag);
+    }
 
-    i = (long) newtListboxGetCurrent(listBox);
-    *result = itemInfo[i].tag;
+    newtFormDestroy(form);
+    free(itemInfo);
 
     return rc;
 }
 
 int checkList(const char * text, int height, int width, poptContext optCon,
-		int useRadio, int flags, const char *** selections) {
+		int useRadio, int flags, char *** selections) {
     newtComponent form, okay, tb, subform, answer;
     newtComponent sb = NULL, cancel = NULL;
     const char * arg;
@@ -398,7 +409,7 @@ int checkList(const char * text, int height, int width, poptContext optCon,
 	const char * tag;
 	newtComponent comp;
     } * cbInfo = malloc(allocedBoxes * sizeof(*cbInfo));
-    char * cbStates = malloc(allocedBoxes * sizeof(cbStates));
+    char * cbStates = malloc(allocedBoxes * sizeof(*cbStates));
 
     if ( (cbInfo == NULL) || (cbStates == NULL)) return DLG_ERROR;
     if (!(arg = poptGetArg(optCon))) return DLG_ERROR;
@@ -473,40 +484,47 @@ int checkList(const char * text, int height, int width, poptContext optCon,
     addButtons(height, width, form, &okay, &cancel, flags);
 
     answer = newtRunForm(form);
+    *selections = NULL;
     if (answer == cancel)
 	rc = DLG_CANCEL;
     if (answer == NULL)
 	rc = DLG_ESCAPE;
-
-    if (useRadio) {
-	answer = newtRadioGetCurrent(cbInfo[0].comp);
-	*selections = malloc(sizeof(char *) * 2);
-	if (*selections == NULL)
-	    return DLG_ERROR;
-	(*selections)[0] = (*selections)[1] = NULL;
-	for (i = 0; i < numBoxes; i++) 
-	    if (cbInfo[i].comp == answer) {
-		(*selections)[0] = cbInfo[i].tag;
-		break;
+    else {
+	if (useRadio) {
+	    answer = newtRadioGetCurrent(cbInfo[0].comp);
+	    *selections = malloc(sizeof(char *) * 2);
+	    if (*selections == NULL)
+		return DLG_ERROR;
+	    (*selections)[0] = (*selections)[1] = NULL;
+	    for (i = 0; i < numBoxes; i++)
+		if (cbInfo[i].comp == answer) {
+		    (*selections)[0] = strdup(cbInfo[i].tag);
+		    break;
+		}
+	} else {
+	    numSelected = 0;
+	    for (i = 0; i < numBoxes; i++) {
+		if (cbStates[i] != ' ') numSelected++;
 	    }
-    } else {
-	numSelected = 0;
-	for (i = 0; i < numBoxes; i++) {
-	    if (cbStates[i] != ' ') numSelected++;
+
+	    *selections = malloc(sizeof(char *) * (numSelected + 1));
+	    if (*selections == NULL)
+		return DLG_ERROR;
+
+	    numSelected = 0;
+	    for (i = 0; i < numBoxes; i++) {
+		if (cbStates[i] != ' ')
+		    (*selections)[numSelected++] = strdup(cbInfo[i].tag);
+	    }
+
+	    (*selections)[numSelected] = NULL;
 	}
-
-	*selections = malloc(sizeof(char *) * (numSelected + 1));
-	if (*selections == NULL) return DLG_ERROR;
-
-	numSelected = 0;
-	for (i = 0; i < numBoxes; i++) {
-	    if (cbStates[i] != ' ') 
-		(*selections)[numSelected++] = cbInfo[i].tag;
-	}
-
-	(*selections)[numSelected] = NULL;
     }
 
+    free(cbInfo);
+    free(cbStates);
+    newtFormDestroy(form);
+
     return rc;
 }
 
@@ -557,8 +575,8 @@ int messageBox(const char * text, int height, int width, int type, int flags) {
 	newtDrawForm(form);
 	newtRefresh();
     }
-	
 
+    newtFormDestroy(form);
 
     return rc;
 }
