@@ -23,8 +23,12 @@ struct entry {
     int firstChar;		/* first character position being shown */
     newtEntryFilter filter;
     void * filterData;
+    int cs;
+    int csDisabled;
 };
 
+static int previous_char(const char *buf, int pos);
+static int next_char(const char *buf, int pos);
 static void entryDraw(newtComponent co);
 static void entryDestroy(newtComponent co);
 static struct eventResult entryEvent(newtComponent co,
@@ -76,6 +80,7 @@ newtComponent newtEntry(int left, int top, const char * initialValue, int width,
     co->width = width;
     co->isMapped = 0;
     co->callback = NULL;
+    co->destroyCallback = NULL;
 
     co->ops = &entryOps;
 
@@ -103,48 +108,74 @@ newtComponent newtEntry(int left, int top, const char * initialValue, int width,
 	strcpy(en->buf, initialValue);
 	en->bufUsed = strlen(initialValue);
 	en->cursorPosition = en->bufUsed;
+
+	/* move cursor back if entry is full */
+	if (en->cursorPosition && !(en->flags & NEWT_FLAG_SCROLL ||
+		    wstrlen(en->buf, -1) < co->width))
+	    en->cursorPosition = previous_char(en->buf, en->cursorPosition);
     } else {
 	*en->buf = '\0';
 	en->bufUsed = 0;
 	en->cursorPosition = 0;
     }
 
+    en->cs = NEWT_COLORSET_ENTRY;
+    en->csDisabled = NEWT_COLORSET_DISENTRY;
+
     return co;
 }
 
-static int visible_width(const char *str, int start, int end)
-{
-    int width = wstrlen(str + start, end-start);
-    int len, w = 0;
-    wchar_t wc;
-
-    len = mbtowc(&wc, str+end, MB_CUR_MAX);
-    if (len == 0)
-       w = 1;
-    else if (len > 0)
-       w = wcwidth(wc);
-    return width + w;
-}
-
 static void scroll(struct entry *en, int width)
 {
-    wchar_t wc;
-    int len, w;
-    int  newwidth = visible_width(en->buf, en->firstChar, en->cursorPosition);
+    int r, lv, rv, cntx, cw, cn, nc, pc, ncw, pcw;
 
-    while (newwidth > width) {
-        len = mbtowc(&wc, en->buf+en->firstChar, MB_CUR_MAX);
-	if (!len) {
-		en->firstChar++;
-		break;
-	}
-	if (len < 0)
-	   break;
-	w = wcwidth(wc);
-	if (w < 0)
-	   break;
-	en->firstChar += len;
-	newwidth -= w;
+    if (width <= 1) {
+	en->firstChar = en->cursorPosition;
+	return;
+    }
+
+    cntx = width / 4;
+    if (cntx > 5)
+	cntx = 5;
+
+    if (en->cursorPosition < en->firstChar)
+	en->firstChar = en->cursorPosition;
+
+    cn = next_char(en->buf, en->cursorPosition);
+    cw = en->cursorPosition >= en->bufUsed ? 1 :
+	wstrlen(en->buf + en->cursorPosition, cn - en->cursorPosition);
+
+    r = wstrlen(en->buf + cn, -1);
+
+    lv = wstrlen(en->buf + en->firstChar, en->cursorPosition - en->firstChar);
+    rv = width - lv - cw;
+
+#define RC (ncw > 0 && (r > rv && lv - ncw >= cntx && rv < cntx))
+#define LC (pcw > 0 && (r + pcw <= rv || (lv < cntx && rv - pcw >= cntx)))
+
+    nc = next_char(en->buf, en->firstChar);
+    ncw = wstrlen(en->buf + en->firstChar, nc - en->firstChar);
+    if (RC) {
+	do {
+	    lv -= ncw;
+	    rv += ncw;
+	    en->firstChar = nc;
+	    nc = next_char(en->buf, en->firstChar);
+	    ncw = wstrlen(en->buf + en->firstChar, nc - en->firstChar);
+	} while (RC);
+	return;
+    }
+
+    pc = previous_char(en->buf, en->firstChar);
+    pcw = wstrlen(en->buf + pc, en->firstChar - pc);
+    if (LC) {
+	do {
+	    lv += pcw;
+	    rv -= pcw;
+	    en->firstChar = pc;
+	    pc = previous_char(en->buf, en->firstChar);
+	    pcw = wstrlen(en->buf + pc, en->firstChar - pc);
+	} while (LC);
     }
 }
 
@@ -158,9 +189,9 @@ static void entryDraw(newtComponent co) {
     if (!co->isMapped) return;
 
     if (en->flags & NEWT_FLAG_DISABLED)
-	SLsmg_set_color(NEWT_COLORSET_DISENTRY);
+	SLsmg_set_color(en->csDisabled);
     else
-	SLsmg_set_color(NEWT_COLORSET_ENTRY);
+	SLsmg_set_color(en->cs);
 
     if (en->flags & NEWT_FLAG_HIDDEN) {
 	newtGotorc(co->top, co->left);
@@ -171,29 +202,35 @@ static void entryDraw(newtComponent co) {
 	return;
     }
 
-    newtGotorc(co->top, co->left);
+    newtTrashScreen();
 
-    if (en->cursorPosition < en->firstChar) {
-	/* scroll to the left */
-	en->firstChar = en->cursorPosition;
-    } else {
-	/* scroll to the right */
-	scroll(en, co->width);
-    }
+    /* scroll if necessary */
+    scroll(en, co->width);
 
     chptr = en->buf + en->firstChar;
 
     if (en->flags & NEWT_FLAG_PASSWORD) {
-	char *p;
-	tmpptr = alloca(strlen(chptr)+2);
-	strcpy(tmpptr, chptr);
-	for (p = tmpptr; *p; p++)
-	    *p = '*';
+	len = wstrlen(chptr, -1);
+	tmpptr = alloca(len + 1);
+	for (i = 0; i < len; i++)
+	    memset(tmpptr, '*', len);
+	tmpptr[len] = '\0';
 	chptr = tmpptr;
     }			
 
     len = wstrlen(chptr, -1);
 
+    /* workaround for double width characters */
+    if (co->width > 1) {
+	i = len < co->width ? len : co->width;
+	i = i > 2 ? i - 2 : 0;
+	newtGotorc(co->top, co->left + i);
+	SLsmg_write_char('_');
+	SLsmg_write_char('_');
+    }
+
+    newtGotorc(co->top, co->left);
+
     if (len <= co->width) {
 	i = len;
 	SLsmg_write_string(chptr);
@@ -201,14 +238,10 @@ static void entryDraw(newtComponent co) {
 	    SLsmg_write_char('_');
 	    i++;
 	}
-    } else {
+    } else
 	SLsmg_write_nstring(chptr, co->width);
-    }
 
-    if (en->flags & NEWT_FLAG_HIDDEN)
-	newtGotorc(co->top, co->left);
-    else
-	newtGotorc(co->top, co->left + wstrlen(en->buf+en->firstChar, en->cursorPosition - en->firstChar));
+    newtGotorc(co->top, co->left + wstrlen(en->buf+en->firstChar, en->cursorPosition - en->firstChar));
 }
 
 void newtEntrySetFlags(newtComponent co, int flags, enum newtFlagsSense sense) {
@@ -227,6 +260,14 @@ void newtEntrySetFlags(newtComponent co, int flags, enum newtFlagsSense sense) {
     newtGotorc(row, col);
 }
 
+void newtEntrySetColors(newtComponent co, int normal, int disabled) {
+    struct entry * en = co->data;
+
+    en->cs = normal;
+    en->csDisabled = disabled;
+    entryDraw(co);
+}
+
 static void entryDestroy(newtComponent co) {
     struct entry * en = co->data;
 
@@ -241,6 +282,8 @@ static struct eventResult entryEvent(newtComponent co,
     struct eventResult er;
     int ch;
 
+    er.result = ER_IGNORED;
+
     if (ev.when == EV_NORMAL) {
 	switch (ev.event) {
 	case EV_FOCUS:
@@ -283,8 +326,7 @@ static struct eventResult entryEvent(newtComponent co,
 	    }
 	    break;
 	}
-    } else
-	er.result = ER_IGNORED;
+    }
 
     return er;
 }
@@ -306,22 +348,21 @@ static int previous_char(const char *buf, int pos)
 static int next_char(const char *buf, int pos)
 {
     int len = mblen(buf + pos, MB_CUR_MAX);
-    if (len < 0)
+    if (len <= 0)
        return pos;
-    if (len == 0)
-       return ++pos;
     return pos+len;
 }
 
 static struct eventResult entryHandleKey(newtComponent co, int key) {
     struct entry * en = co->data;
     struct eventResult er;
-    char * chptr, * insPoint;
+    char * chptr;
 
     er.result = ER_SWALLOWED;
     switch (key) {
       case '\r':				/* Return */
 	if (en->flags & NEWT_FLAG_RETURNEXIT) {
+	    newtCursorOff();
 	    er.result = ER_EXITFORM;
 	} else {
 	    er.result = ER_NEXTCOMP;
@@ -398,47 +439,55 @@ static struct eventResult entryHandleKey(newtComponent co, int key) {
 
       default:
 	if ((key >= 0x20 && key <= 0x7e) || (key >= 0x80 && key <= 0xff)) {
-	    if (!(en->flags & NEWT_FLAG_SCROLL) && wstrlen(en->buf, -1) >= co->width) {
+	    char s[MB_CUR_MAX];
+	    mbstate_t ps;
+	    int i, l;
+
+	    for (i = 1, s[0] = key; ; i++) {
+		memset(&ps, 0, sizeof (ps));
+		l = mbrtowc(NULL, s, i, &ps);
+		if (l == -1)	/* invalid sequence */
+		    i = 0;
+		if (l != -2)	/* not incomplete sequence */
+		    break;
+
+		/* read next byte */
+		if (i == MB_CUR_MAX || !SLang_input_pending(1)) {
+		    i = 0;
+		    break;
+		}
+		s[i] = SLang_getkey();
+	    }
+
+	    if (!i || (!(en->flags & NEWT_FLAG_SCROLL) && wstrlen(en->buf, -1) + wstrlen(s, i) > co->width)) {
 		/* FIXME this is broken */
 		SLtt_beep();
 		break;
 	    }
 
-	    if ((en->bufUsed + 1) == en->bufAlloced) {
+	    if ((en->bufUsed + i) >= en->bufAlloced) {
 		en->bufAlloced += 20;
 		en->buf = realloc(en->buf, en->bufAlloced);
 		if (en->resultPtr) *en->resultPtr = en->buf;
-		memset(en->buf + en->bufUsed + 1, 0, 20);
+		memset(en->buf + en->bufAlloced - 20, 0, 20);
 	    }
 
-	    if (en->cursorPosition == en->bufUsed) {
-		en->bufUsed++;
-	    } else {
+	    if (en->cursorPosition != en->bufUsed) {
 		/* insert the new character */
-
-		/* chptr is the last character in the string */
-		chptr = (en->buf + en->bufUsed) - 1;
-		if ((en->bufUsed + 1) == en->bufAlloced) {
-		    /* this string fills the buffer, so clip it */
-		    chptr--;
-		} else
-		    en->bufUsed++;
-
-		insPoint = en->buf + en->cursorPosition;
-
-		while (chptr >= insPoint) {
-		    *(chptr + 1) = *chptr;
-		    chptr--;
-		}
-
+		memmove(en->buf + en->cursorPosition + i, en->buf + en->cursorPosition, en->bufUsed - en->cursorPosition);
 	    }
-            /* FIXME */
-	    en->buf[en->cursorPosition++] = key;
+	    en->bufUsed += i;
+	    for (l = 0; l < i; l++)
+		en->buf[en->cursorPosition++] = s[l];
 	} else {
 	    er.result = ER_IGNORED;
 	}
     }
 
+    if (en->cursorPosition == en->bufUsed && en->cursorPosition &&
+	    !(en->flags & NEWT_FLAG_SCROLL || wstrlen(en->buf, -1) < co->width))
+	en->cursorPosition = previous_char(en->buf, en->cursorPosition);
+
     entryDraw(co);
 
     return er;
