@@ -74,7 +74,7 @@ static void listboxPlace(newtComponent co, int newLeft, int newTop) {
 
     if (li->sb)
 	li->sb->ops->place(li->sb, co->left + co->width - li->bdxAdjust - 1,
-			   co->top);
+			   co->top + li->bdyAdjust);
 }
 
 newtComponent newtListbox(int left, int top, int height, int flags) {
@@ -100,7 +100,7 @@ newtComponent newtListbox(int left, int top, int height, int flags) {
     li->bdxAdjust = 0;
     li->bdyAdjust = 0;
     li->flags = flags & (NEWT_FLAG_RETURNEXIT | NEWT_FLAG_BORDER |
-			 NEWT_FLAG_MULTIPLE);
+			 NEWT_FLAG_MULTIPLE | NEWT_FLAG_SHOWCURSOR);
 
     if (li->flags & NEWT_FLAG_BORDER) {
 	li->bdxAdjust = 2;
@@ -133,6 +133,7 @@ newtComponent newtListbox(int left, int top, int height, int flags) {
     co->ops = &listboxOps;
     co->takesFocus = 1;
     co->callback = NULL;
+    co->destroyCallback = NULL;
 
     updateWidth(co, li, 5);
 
@@ -200,7 +201,8 @@ void newtListboxSetWidth(newtComponent co, int width) {
     co->width = width;
     li->curWidth = co->width - li->sbAdjust - 2 * li->bdxAdjust;
     li->userHasSetWidth = 1;
-    if (li->sb) li->sb->left = co->width + co->left - 1;
+    if (li->sb)
+	li->sb->left = co->left + co->width - li->bdxAdjust - 1;
     listboxDraw(co);
 }
 
@@ -297,8 +299,8 @@ void newtListboxSetEntry(newtComponent co, int num, const char * text) {
 	free(item->text);
 	item->text = strdup(text);
     }
-    if (li->userHasSetWidth == 0 && strlen(text) > li->curWidth) {
-	updateWidth(co, li, strlen(text));
+    if (li->userHasSetWidth == 0 && wstrlen(text,-1) > li->curWidth) {
+	updateWidth(co, li, wstrlen(text,-1));
     }
 
     if (num >= li->startShowItem && num <= li->startShowItem + co->height)
@@ -313,7 +315,8 @@ void newtListboxSetData(newtComponent co, int num, void * data) {
     for(i = 0, item = li->boxItems; item != NULL && i < num;
 	i++, item = item->next);
 
-    item->data = data;
+    if (item)
+	item->data = data;
 }
 
 int newtListboxAppendEntry(newtComponent co, const char * text,
@@ -329,8 +332,8 @@ int newtListboxAppendEntry(newtComponent co, const char * text,
 	item = li->boxItems = malloc(sizeof(struct items));
     }
 
-    if (!li->userHasSetWidth && text && (strlen(text) > li->curWidth))
-	updateWidth(co, li, strlen(text));
+    if (!li->userHasSetWidth && text && (wstrlen(text,-1) > li->curWidth))
+	updateWidth(co, li, wstrlen(text,-1));
 
     item->text = strdup(text); item->data = data; item->next = NULL;
     item->isSelected = 0;
@@ -369,8 +372,8 @@ int newtListboxInsertEntry(newtComponent co, const char * text,
 	item->next = NULL;
     }
 
-    if (!li->userHasSetWidth && text && (strlen(text) > li->curWidth))
-	updateWidth(co, li, strlen(text));
+    if (!li->userHasSetWidth && text && (wstrlen(text,-1) > li->curWidth))
+	updateWidth(co, li, wstrlen(text,-1));
 
     item->text = strdup(text?text:"(null)"); item->data = data;
     item->isSelected = 0;
@@ -417,7 +420,7 @@ int newtListboxDeleteEntry(newtComponent co, void * key) {
     if (!li->userHasSetWidth) {
 	widest = 0;
 	for (item = li->boxItems; item != NULL; item = item->next)
-	    if ((t = strlen(item->text)) > widest) widest = t;
+	    if ((t = wstrlen(item->text,-1)) > widest) widest = t;
     }
 
     if (li->currItem >= num)
@@ -449,6 +452,12 @@ void newtListboxClear(newtComponent co)
 	updateWidth(co, li, 5);
 }
 
+int newtListboxItemCount(newtComponent co)
+{
+    struct listbox *li = co->data;
+    return li->numItems;
+}
+
 /* If you don't want to get back the text, pass in NULL for the ptr-ptr. Same
    goes for the data. */
 void newtListboxGetEntry(newtComponent co, int num, char **text, void **data) {
@@ -486,6 +495,8 @@ static void listboxDraw(newtComponent co)
 
     if (!co->isMapped) return ;
 
+    newtTrashScreen();
+    
     if(li->flags & NEWT_FLAG_BORDER) {
       if(li->isActive)
 	  SLsmg_set_color(NEWT_COLORSET_ACTLISTBOX);
@@ -510,7 +521,7 @@ static void listboxDraw(newtComponent co)
 
 	newtGotorc(co->top + i + li->bdyAdjust, co->left + li->bdxAdjust);
 	if(j + i == li->currItem) {
-	    if(item->isSelected)
+	    if(li->isActive)
 		SLsmg_set_color(NEWT_COLORSET_ACTSELLISTBOX);
 	    else
 		SLsmg_set_color(NEWT_COLORSET_ACTLISTBOX);
@@ -521,8 +532,15 @@ static void listboxDraw(newtComponent co)
 
 	SLsmg_write_nstring(item->text, li->curWidth);
 
+	if (li->flags & NEWT_FLAG_MULTIPLE) {
+	    newtGotorc(co->top + i + li->bdyAdjust, co->left + li->bdxAdjust);
+	    SLsmg_set_color(item->isSelected ?
+		    NEWT_COLORSET_SELLISTBOX : NEWT_COLORSET_LISTBOX);
+	    SLsmg_write_nstring(item->text, 1);
+	}
     }
-    newtGotorc(co->top + (li->currItem - li->startShowItem), co->left);
+    newtGotorc(co->top + (li->currItem - li->startShowItem) + li->bdyAdjust,
+               co->left + li->bdxAdjust);
 }
 
 static struct eventResult listboxEvent(newtComponent co, struct event ev) {
@@ -544,7 +562,7 @@ static struct eventResult listboxEvent(newtComponent co, struct event ev) {
 	switch(ev.u.key) {
 	  case ' ':
 	    if(!(li->flags & NEWT_FLAG_MULTIPLE)) break;
-	    newtListboxSelectItem(co, li->boxItems[li->currItem].data,
+	    newtListboxSelectItem(co, newtListboxGetCurrent(co),
 				  NEWT_FLAGS_TOGGLE);
 	    er.result = ER_SWALLOWED;
 	    /* We don't break here, because it is cool to be able to
@@ -652,7 +670,7 @@ static struct eventResult listboxEvent(newtComponent co, struct event ev) {
 			 li->currItem > li->startShowItem)
 			  li->startShowItem =
 			      li->currItem > li->numItems - li->curHeight ?
-			      li->startShowItem = li->numItems - li->curHeight :
+			      li->numItems - li->curHeight :
 			      li->currItem;
 		      if(li->sb)
 			  newtScrollbarSet(li->sb, li->currItem + 1, li->numItems);
@@ -666,12 +684,16 @@ static struct eventResult listboxEvent(newtComponent co, struct event ev) {
       case EV_FOCUS:
 	li->isActive = 1;
 	listboxDraw(co);
+	if(li->flags & NEWT_FLAG_SHOWCURSOR)
+	  newtCursorOn();
 	er.result = ER_SWALLOWED;
 	break;
 
       case EV_UNFOCUS:
 	li->isActive = 0;
 	listboxDraw(co);
+	if(li->flags & NEWT_FLAG_SHOWCURSOR)
+	  newtCursorOff();
 	er.result = ER_SWALLOWED;
 	break;
 
