@@ -54,7 +54,7 @@ RIGHT = (1, 0)
 snackArgs = {"append":-1}
 
 class Widget:
-    """Base class for NEWT toolkit
+    """Base class for NEWT toolkit - Do not use directly
 
     methods:
 
@@ -70,7 +70,7 @@ class Widget:
             self.w.setCallback(obj)
             
     def __init__(self):
-        self.w = None
+        raise NotImplementedError
 
 class Button(Widget):
     """Basic button class, takes button text as parameter
@@ -144,11 +144,12 @@ class Listbox(Widget):
 
     methods:
 
-     - Listbox(self,height, scroll = 0l returnExit = 0, width = 0, showCursor = 0)
+     - Listbox(self, height, scroll = 0, returnExit = 0, width = 0, showCursor = 0, multiple = 0, border = 0)
      - insert(self, text, item, before) : insert element; before = key to item to insert before, or None.
      - delete(self, item)               : delete item from list.
      - replace(self, text,item)         : Replace a given item's text
      - current(self)                    : returns currently selected item
+     - getSelection(self)               : returns a list of selected items
      - setCurrent(self,i tem)           : select current.
      - clear(self)                      : clear listbox
     """
@@ -181,6 +182,13 @@ class Listbox(Widget):
     def current(self):
         return self.key2item[self.w.listboxGetCurrent()]
 
+    def getSelection(self):
+        selection = []
+        list = self.w.listboxGetSelection()
+        for key in list:
+            selection.append(self.key2item[key])
+        return selection
+
     def setCurrent(self, item):
         self.w.listboxSetCurrent(self.item2key[item])
 
@@ -189,8 +197,8 @@ class Listbox(Widget):
         self.item2key = {}        
         self.w.listboxClear()
 
-    def __init__(self, height, scroll = 0, returnExit = 0, width = 0, showCursor = 0):
-        self.w = _snack.listbox(height, scroll, returnExit, showCursor)
+    def __init__(self, height, scroll = 0, returnExit = 0, width = 0, showCursor = 0, multiple = 0, border = 0):
+        self.w = _snack.listbox(height, scroll, returnExit, showCursor, multiple, border)
         self.key2item = {}
         self.item2key = {}
         if (width):
@@ -204,11 +212,15 @@ class Textbox(Widget):
      - Textbox(self, width, height, scroll = 0, wrap = 0): scroll, wrap are flags
                                    include scroll bars, or text wrap.
      - setText(text) : set text.
+     - setHeight(height): set height.
     """
     
     def setText(self, text):
         self.w.textboxText(text)
 
+    def setHeight(self, height):
+        self.w.textboxHeight(height)
+
     def __init__(self, width, height, text, scroll = 0, wrap = 0):
         self.w = _snack.textbox(width, height, text, scroll, wrap)
 
@@ -228,6 +240,7 @@ class Label(Widget):
 
      - Label(self,text)   : create label
      - setText(self,text) : change text.
+     - setColors(self, colorset) : change individual colors
     """
     def setText(self, text):
         self.w.labelText(text)
@@ -235,6 +248,9 @@ class Label(Widget):
     def __init__(self, text):
         self.w = _snack.label(text)
 
+    def setColors(self, colorset):
+        self.w.labelSetColors(colorset)
+
 class Scale(Widget):
     """A Scale (progress bar).
 
@@ -260,14 +276,14 @@ class Entry(Widget):
           if returnExit is set, return from Form when exiting this element, else
            proceed to next entry widget.
      - value(self): return value.
-     - set(text) : set the text
+     - set(text, cursorAtEnd = 1) : set the text
      - setFlags (flag, sense) : flags can be FLAG_DISABLED, FLAGS_SET, FLAGS_RESET, FLAGS_TOGGLE
     """
     def value(self):
         return self.w.entryValue
 
-    def set(self, text):
-        return self.w.entrySetValue(text)
+    def set(self, text, cursorAtEnd = 1):
+        return self.w.entrySetValue(text, cursorAtEnd)
 
     def setFlags (self, flag, sense):
         return self.w.entrySetFlags(flag, sense)
@@ -282,10 +298,17 @@ hotkeys = { "F1" : _snack.KEY_F1, "F2" : _snack.KEY_F2, "F3" : _snack.KEY_F3,
             "F4" : _snack.KEY_F4, "F5" : _snack.KEY_F5, "F6" : _snack.KEY_F6, 
             "F7" : _snack.KEY_F7, "F8" : _snack.KEY_F8, "F9" : _snack.KEY_F9, 
             "F10" : _snack.KEY_F10, "F11" : _snack.KEY_F11, 
-            "F12" : _snack.KEY_F12, "ESC" : _snack.KEY_ESC ,  " " : ord(" ") }
+            "F12" : _snack.KEY_F12, "ESC" : _snack.KEY_ESC,
+            "ENTER": _snack.KEY_ENTER, "SUSPEND" : _snack.KEY_SUSPEND,
+            "BACKSPACE": _snack.KEY_BACKSPACE, "DELETE": _snack.KEY_DELETE,
+            "INSERT": _snack.KEY_INSERT,
+             " " : ord(" ") }
 
 for n in hotkeys.keys():
     hotkeys[hotkeys[n]] = n
+for o,c in [ (ord(c),c) for c in string.ascii_letters+string.digits ]:
+    hotkeys[c] = o
+    hotkeys[o] = c
 
 class Form:
     """ Base Form class, from which Grid, etc. inherit
@@ -395,6 +418,30 @@ class Grid:
         self.g = apply(_snack.grid, args)
         self.gridmembers = []
 
+colorsets = { "ROOT" : _snack.COLORSET_ROOT,
+              "BORDER" : _snack.COLORSET_BORDER,
+              "WINDOW" : _snack.COLORSET_WINDOW,
+              "SHADOW" : _snack.COLORSET_SHADOW,
+              "TITLE" : _snack.COLORSET_TITLE,
+              "BUTTON" : _snack.COLORSET_BUTTON,
+              "ACTBUTTON" : _snack.COLORSET_ACTBUTTON,
+              "CHECKBOX" : _snack.COLORSET_CHECKBOX,
+              "ACTCHECKBOX" : _snack.COLORSET_ACTCHECKBOX,
+              "ENTRY" : _snack.COLORSET_ENTRY,
+              "LABEL" : _snack.COLORSET_LABEL,
+              "LISTBOX" : _snack.COLORSET_LISTBOX,
+              "ACTLISTBOX" : _snack.COLORSET_ACTLISTBOX,
+              "TEXTBOX" : _snack.COLORSET_TEXTBOX,
+              "ACTTEXTBOX" : _snack.COLORSET_ACTTEXTBOX,
+              "HELPLINE" : _snack.COLORSET_HELPLINE,
+              "ROOTTEXT" : _snack.COLORSET_ROOTTEXT,
+              "EMPTYSCALE" : _snack.COLORSET_EMPTYSCALE,
+              "FULLSCALE" : _snack.COLORSET_FULLSCALE,
+              "DISENTRY" : _snack.COLORSET_DISENTRY,
+              "COMPACTBUTTON" : _snack.COLORSET_COMPACTBUTTON,
+              "ACTSELLISTBOX" : _snack.COLORSET_ACTSELLISTBOX,
+              "SELLISTBOX" : _snack.COLORSET_SELLISTBOX }
+
 class SnackScreen:
     """A Screen;
 
@@ -409,6 +456,11 @@ class SnackScreen:
     - suspendcallback(self,cb, data=None) : set callback. data=data to pass to cb.
     - openWindow(self,left, top, width, height, title): Open a window.
     - pushHelpLine(self,text): put help line on screen. Returns current help line if text=None
+    - setColor(self, colorset, fg, bg): Set foreground and background colors;
+            colorset = key from snack.colorsets,
+            fg & bg = english color names defined by S-Lang
+                (ref: S-Lang Library C Programmer's Guide section:
+                8.4.4.  Setting Character Attributes)
     """
     def __init__(self):
         _snack.init()
@@ -428,7 +480,7 @@ class SnackScreen:
         self.helpCb(self, arg)
     
     def helpCallback(self, cb):
-    	self.helpCb = cb
+        self.helpCb = cb
         return _snack.helpcallback(self.doHelpCallback)
 
     def suspendCallback(self, cb, data = None):
@@ -460,12 +512,21 @@ class SnackScreen:
 
         return _snack.gridwrappedwindow(grid.g, title)
 
-    def popWindow(self):
-        return _snack.popwindow()
+    def popWindow(self, refresh = True):
+        if refresh:
+            return _snack.popwindow()
+        return _snack.popwindownorefresh()
 
     def refresh(self):
         return _snack.refresh()
 
+    def setColor(self, colorset, fg, bg):
+        if colorset in colorsets:
+            return _snack.setcolor(colorsets[colorset], fg, bg)
+        else:
+           # assume colorset is an integer for the custom color set
+           return _snack.setcolor(colorset, fg, bg)
+
 def reflow(text, width, flexDown = 5, flexUp = 5):
     """ returns a tuple of the wrapped text, the actual width, and the actual height
     """
@@ -566,7 +627,7 @@ class ButtonBar(Grid):
 
         for (button, value) in self.list:
             if result == button:
-            	return value
+                return value
         return None
 
 
@@ -789,6 +850,8 @@ def EntryWindow(screen, title, text, prompts, allowCancel = 1, width = 40,
     for n in prompts:
         if (type(n) == types.TupleType):
             (n, e) = n
+            if (type(e) in types.StringTypes):
+                e = Entry(entryWidth, e)
         else:
             e = Entry(entryWidth)
 
@@ -927,3 +990,6 @@ class CListbox(Grid):
 
     def clear(self):
         self.listbox.clear()
+
+def customColorset(x):
+    return 30 + x
